@@ -5,9 +5,8 @@
 use std::fs;
 use std::io::Write;
 
-use adblock::engine::Engine;
 use adblock::{
-    blocker::Redirection,
+    engine::Engine,
     lists::{FilterSet, ParseOptions},
 };
 
@@ -56,7 +55,7 @@ fn new_adblock(list_dir: &str) -> Box<Adblock> {
 }
 
 fn load_adblock(path: &str) -> Box<Adblock> {
-    let engine = match std::fs::read(path) {
+    let engine = match fs::read(path) {
         Ok(data) => {
             let mut engine = Engine::new(true);
             match engine.deserialize(&data) {
@@ -91,11 +90,8 @@ impl Adblock {
             return ffi::AdblockResult {
                 matched: blocker_result.matched,
                 important: blocker_result.important,
-                redirect: match blocker_result.redirect {
-                    Some(Redirection::Url(url)) => url,
-                    Some(Redirection::Resource(data_url)) => data_url, // Seems to contain a data:text/plain;base64 url
-                    _ => String::new(),
-                },
+                redirect: blocker_result.redirect.unwrap_or_default(),
+                rewritten_url: blocker_result.rewritten_url.unwrap_or_default(),
             };
         } else {
             adblock_debug!("Adblock engine doesn't exist! Probably it failed to load or restore");
@@ -104,15 +100,21 @@ impl Adblock {
         ffi::AdblockResult::default()
     }
 
-    fn get_cosmetic_filters(&self, url: &str, classes: &[String], ids: &[String]) -> Vec<String> {
-        if let Some(engine) = &self.blocker {
-            let cosmetic_resources = engine.url_cosmetic_resources(url);
-            let selectors =
-                engine.hidden_class_id_selectors(classes, ids, &cosmetic_resources.exceptions);
-            return selectors;
-        }
+    fn get_injected_script(&self, url: &str) -> String {
+        self.blocker
+            .as_ref()
+            .map(|engine| engine.url_cosmetic_resources(url).injected_script)
+            .unwrap_or_default()
+    }
 
-        Vec::new()
+    fn get_cosmetic_filters(&self, url: &str, classes: &[String], ids: &[String]) -> Vec<String> {
+        self.blocker
+            .as_ref()
+            .map(|engine| {
+                let cosmetic_resources = engine.url_cosmetic_resources(url);
+                engine.hidden_class_id_selectors(classes, ids, &cosmetic_resources.exceptions)
+            })
+            .unwrap_or_default()
     }
 
     fn save(&self, path: &str) -> bool {
@@ -152,6 +154,8 @@ mod ffi {
         matched: bool,
         important: bool,
         redirect: String,
+        #[cxx_name = "rewrittenUrl"]
+        rewritten_url: String,
     }
 
     extern "Rust" {
@@ -180,6 +184,8 @@ mod ffi {
             classes: &[String],
             ids: &[String],
         ) -> Vec<String>;
+        #[cxx_name = "getInjectedScript"]
+        fn get_injected_script(self: &Adblock, url: &str) -> String;
         fn save(self: &Adblock, path: &str) -> bool;
     }
 }
